@@ -1,6 +1,6 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
-    SPDX-FileCopyrightText: 2023-2024 Louis Schul <schul9louis@gmail.com>
+    SPDX-FileCopyrightText: 2023-2025 Louis Schul <schul9louis@gmail.com>
 */
 
 #include "renderer.h"
@@ -8,6 +8,7 @@
 #include <QDir>
 #include <QRegularExpression>
 #include <QUrl>
+#include <qstringliteral.h>
 
 Renderer::Renderer()
     : MD::details::HtmlVisitor<MD::QStringTrait>() {};
@@ -81,7 +82,7 @@ void Renderer::onHeading(
     if (!m_justCollectFootnoteRefs) {
         m_html.push_back(QStringLiteral("<"));
         m_html.push_back(ht);
-        m_html.push_back(MD::details::headingIdToHtml(h));
+        m_html.push_back(headingIdToHtml(h));
         m_html.push_back(QStringLiteral(">"));
     }
 
@@ -138,11 +139,11 @@ void Renderer::onLink(MD::Link<MD::QStringTrait> *l)
         }
     } else if (!l->text().isEmpty()) {
         if (!m_justCollectFootnoteRefs) {
-            m_html.push_back(MD::details::prepareTextForHtml<MD::QStringTrait>(l->text()));
+            m_html.push_back(prepareTextForHtml(l->text()));
         }
     } else {
         if (!m_justCollectFootnoteRefs) {
-            m_html.push_back(MD::details::prepareTextForHtml<MD::QStringTrait>(l->url()));
+            m_html.push_back(prepareTextForHtml(l->url()));
         }
     }
 
@@ -158,7 +159,7 @@ void Renderer::onImage(MD::Image<MD::QStringTrait> *i)
     if (!m_justCollectFootnoteRefs) {
         QString url = i->url();
         if (url.startsWith(QStringLiteral("./"))) {
-            url = m_notePath + url.mid(1);
+            url = m_noteDir + url.mid(1);
         }
         if (url.startsWith(QStringLiteral("~"))) {
             url = QDir::homePath() + url.mid(1);
@@ -253,9 +254,14 @@ void Renderer::onEmoji(EmojiPlugin::EmojiItem *e)
 
 // Internal info
 // =============
-void Renderer::setNotePath(const QString &notePath)
+QString Renderer::getNoteDir() const
 {
-    m_notePath = notePath;
+    return m_noteDir;
+}
+
+void Renderer::setNoteDir(const QString &noteDir)
+{
+    m_noteDir = noteDir;
 }
 
 void Renderer::addPluginHelper(PluginHelper *pluginHelper)
@@ -291,7 +297,19 @@ void Renderer::setCodeHighlightEnable(const bool enable)
 // =========
 QString Renderer::code(QString &code)
 {
-    return QStringLiteral("<pre><code>") + code + QStringLiteral("</code></pre>\n");
+    static const QString buttonStyle = QStringLiteral(
+        "class=\"klever-copy-button\" style=\"width:2.2em; height:2.2em; position:absolute; top: 5px; right: 5px; display: flex; justify-content: center; "
+        "align-items: center; padding: 0.2em\"");
+    static const QString onClick = QStringLiteral("onClick=\"copyCode(this)\">");
+    static const QString svgPath = QStringLiteral(
+        "<path fill-rule=\"evenodd\" d=\"M4 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2zm2-1a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h8a1 1 0 0 0 "
+        "1-1V2a1 1 0 0 0-1-1zM2 5a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1v-1h1v1a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V6a2 2 0 0 1 2-2h1v1z\"/>");
+    static const QString svg =
+        QStringLiteral("<svg style=\"width: 1.2em; height: 1.2em\" fill=\"currentColor\"> viewBox=\"0 0 16 16\"") + svgPath + QStringLiteral("</svg>");
+
+    static const QString button = QStringLiteral("<button ") + buttonStyle + onClick + svg + QStringLiteral("</button>");
+
+    return QStringLiteral("<pre style=\"position:relative\">") + button + QStringLiteral("<code>") + code + QStringLiteral("</code></pre>\n");
 }
 
 QString Renderer::openListItem(const bool hasTask, const bool isChecked, const int startNumber)
@@ -302,12 +320,12 @@ QString Renderer::openListItem(const bool hasTask, const bool isChecked, const i
     }
 
     const QString checkboxStr = checkbox(isChecked);
-    return QStringLiteral("<li class=\"hasCheck\"> <label class=\"form-control\">\n") + checkboxStr;
+    return QStringLiteral("<li class=\"hasCheck\"> <label class=\"form-control\">\n") + checkboxStr + QStringLiteral("<span>");
 }
 
 QString Renderer::closeListItem(const bool hasTask)
 {
-    return hasTask ? QStringLiteral("</label></li>\n") : QLatin1String();
+    return hasTask ? QStringLiteral("</span></label></li>\n") : QStringLiteral("</li>\n");
 }
 
 QString Renderer::checkbox(bool checked)
