@@ -1,6 +1,6 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
-    SPDX-FileCopyrightText: 2022 Louis Schul <schul9louis@gmail.com>
+    SPDX-FileCopyrightText: 2022-2025 Louis Schul <schul9louis@gmail.com>
 */
 
 #include <QApplication>
@@ -14,6 +14,7 @@
 #include "app.h"
 #include "kleverconfig.h"
 #include <KAboutData>
+#include <KIconTheme>
 #include <KLocalizedContext>
 #include <KLocalizedString>
 
@@ -24,14 +25,7 @@
 
 #include "logic/editor/editorHandler.hpp"
 
-#include "logic/treeview/noteTreeModel.h"
-
-#include "logic/painting/pressureequation.h"
-#include "logic/painting/sketchmodel.h"
-#include "logic/painting/sketchserializer.h"
-#include "logic/painting/sketchview.h"
-#include "logic/painting/strokeitem.h"
-#include "logic/painting/strokelistitem.h"
+#include "logic/treeview/treeModel.h"
 
 #include "logic/printing/printingHelper.h"
 
@@ -46,6 +40,10 @@
 Q_DECL_EXPORT int main(int argc, char *argv[])
 {
     QtWebEngineQuick::initialize();
+
+    KIconTheme::initTheme();
+    QIcon::setFallbackThemeName(QStringLiteral("breeze"));
+
     QApplication app(argc, argv);
     QCoreApplication::setOrganizationName(QStringLiteral("KDE"));
 
@@ -57,7 +55,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
         // A displayable program name string.
         i18nc("@title", "KleverNotes"),
         // The program version string.
-        QStringLiteral("1.1.0"),
+        QStringLiteral("1.2.0"),
         // Short description of what the app does.
         i18n("Note taking and management application"),
         // The license this code is released under.
@@ -78,8 +76,7 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     auto config = KleverConfig::self();
     // General
     QObject::connect(config, &KleverConfig::storagePathChanged, config, &KleverConfig::save);
-    QObject::connect(config, &KleverConfig::defaultCategoryNameChanged, config, &KleverConfig::save);
-    QObject::connect(config, &KleverConfig::defaultGroupNameChanged, config, &KleverConfig::save);
+    QObject::connect(config, &KleverConfig::defaultFolderNameChanged, config, &KleverConfig::save);
     QObject::connect(config, &KleverConfig::defaultNoteNameChanged, config, &KleverConfig::save);
     QObject::connect(config, &KleverConfig::editorVisibleChanged, config, &KleverConfig::save);
     QObject::connect(config, &KleverConfig::previewVisibleChanged, config, &KleverConfig::save);
@@ -87,8 +84,6 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     QObject::connect(config, &KleverConfig::spacesForTabChanged, config, &KleverConfig::save);
     QObject::connect(config, &KleverConfig::visibleToolsChanged, config, &KleverConfig::save);
     QObject::connect(config, &KleverConfig::invisibleToolsChanged, config, &KleverConfig::save);
-    // Appearence
-    QObject::connect(config, &KleverConfig::categoryDisplayNameChanged, config, &KleverConfig::save);
     // Plugins
     // TODO move a maximum of this to plugin helper or editorHandler
     QObject::connect(config, &KleverConfig::codeSynthaxHighlighterChanged, config, &KleverConfig::save);
@@ -138,29 +133,6 @@ Q_DECL_EXPORT int main(int argc, char *argv[])
     qmlRegisterSingletonInstance<HighlightHelper>("org.kde.Klever", 1, 0, "HighlightHelper", &highlightHelper);
     // === === === ===
 
-    qRegisterMetaType<StrokeSample>();
-    qRegisterMetaType<Stroke>();
-    qRegisterMetaType<Stroke::Type>();
-    qRegisterMetaType<Event>();
-
-    // Will complain about valueType names starting with capital letter, but changing it will break things and make CLazy angry
-    qmlRegisterUncreatableType<StrokeSample>("WashiPadComponents",
-                                             1,
-                                             0,
-                                             "StrokeSample",
-                                             QStringLiteral("Use the createSample function on SketchViewHandler instead"));
-    qmlRegisterUncreatableType<Stroke>("WashiPadComponents", 1, 0, "Stroke", QStringLiteral("Use the createStroke function on SketchViewHandler instead"));
-    qmlRegisterUncreatableType<Event>("WashiPadComponents", 1, 0, "Event", QStringLiteral("They are provided by the SketchViewHandler"));
-
-    qmlRegisterType<PressureEquation>("WashiPadComponents", 1, 0, "PressureEquation");
-
-    qmlRegisterType<SketchViewHandler>("WashiPadComponents", 1, 0, "SketchViewHandler");
-
-    qmlRegisterType<SketchModel>("WashiPadComponents", 1, 0, "SketchModel");
-    qmlRegisterType<SketchSerializer>("WashiPadComponents", 1, 0, "SketchSerializer");
-    qmlRegisterType<StrokeItem>("WashiPadComponents", 1, 0, "StrokeItem");
-    qmlRegisterType<StrokeListItem>("WashiPadComponents", 1, 0, "StrokeListItem");
-
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
     engine.load(QUrl(QStringLiteral("qrc:/contents/ui/main.qml")));
 
